package gomod

import (
	"Geoswift_No_Reason_Remittance_Golang/models"
	"Geoswift_No_Reason_Remittance_Golang/utils"
	"encoding/json"
	"fmt"
	"net/http"
	"path/filepath"
)

func CloseRequest(w http.ResponseWriter, r *http.Request) {
	//接口地址
	apiurl := "https://apis.geoswift.com/remittance/cancel"
	//pfx私钥
	privatepfxpath, pfxerr := filepath.Abs("package/JHclient.pfx")
	if pfxerr != nil {
		fmt.Println("pfxerr:", pfxerr)
	}
	pfxpwd := "1234"
	//cer公钥
	publiccerpath, cererr := filepath.Abs("package/server.cer")
	if cererr != nil {
		fmt.Println("cererr:", cererr)
	}
	//生成16位随机数
	var aesKeyValue string = utils.GetRandomString(16)
	r.ParseForm()
	var close models.Close
	close.MerchantId = utils.IsNull(r.Form["merchantId"])
	close.RequestId = utils.IsNull(r.Form["requestId"])
	close.RemittanceRequestId = utils.IsNull(r.Form["remittanceRequestId"])
	close.ProxyRequestId = utils.IsNull(r.Form["proxyRequestId"])
	close.MerchantReference = utils.IsNull(r.Form["merchantReference"])
	close.Remark = utils.IsNull(r.Form["remark"])

	//把订单对象数据转JSON字符串
	closedata, err := json.Marshal(close)
	if err != nil {
		fmt.Print(err.Error())
		return
	}
	sortdata, oerr := utils.GetMapSortJson(string(closedata))
	if oerr != nil {
		fmt.Println("oerr:", oerr)
	}
	hm := utils.Sha1(sortdata)
	hmac, err := utils.Sign(hm, privatepfxpath, pfxpwd)
	if err != nil {
		fmt.Println("err:", err)
	}
	close.Hmac = hmac
	orderjson, err := json.Marshal(close)
	if err != nil {
		fmt.Printf(err.Error())
		return
	}
	encryptKey, enerr := utils.PublicEncrypt(aesKeyValue, publiccerpath)
	if enerr != nil {
		fmt.Println("err:", enerr)
	}
	aesdata, aeserro := utils.AesECBEncrypt(string(orderjson), aesKeyValue)
	//fmt.Println("aesdata:", aesdata)
	if aeserro != nil {
		fmt.Println("err:", aeserro)
	}
	response, rperr := utils.Post(apiurl, encryptKey, close.MerchantId, close.RequestId, aesdata, privatepfxpath, pfxpwd)
	if rperr != nil {
		fmt.Println("rperr:", rperr)
	}
	var closebacks models.CloseBack
	json.Unmarshal([]byte(response), &closebacks)
	rhmac := closebacks.Hmac
	//fmt.Println("rhmac:", rhmac)
	if rhmac != "" {
		sortdeldata, oerr := utils.GetDelJson(string(response), "hmac")
		if oerr != nil {
			fmt.Println("oerr:", oerr)
		}
		signhm := utils.Sha1(sortdeldata)
		verr := utils.VerifySign(signhm, publiccerpath, rhmac)
		if verr == nil {
			status := closebacks.Status
			fmt.Println("response:", response)
			fmt.Fprintf(w, response)
			if status == "REDIRECT" {
				w.Header().Set("Location", closebacks.RedirectUrl) //跳转地址设置
				w.WriteHeader(301)
				http.Redirect(w, r, closebacks.RedirectUrl, http.StatusFound)
				return
			} else if status == "SUCCESS" {
				//返回扫码Base64不为空
				if closebacks.ScanCodeUrl != "" {
					//fmt.Println("response:", response)
					//fmt.Println("orderbacks.ScanCode:", orderbacks.ScanCode)
					fmt.Fprintf(w, "{\"type\":\"code\",\"val\":\""+closebacks.ScanCodeUrl+"\"}")
					//w.Header().Set("Content-Type", "text/html")
					//fmt.Fprintf(w,"请扫描下方二维码完成支付：<br/>")
					//fmt.Fprintf(w, "<img id=\"cordbase64\" src=\"data:image/png;base64,"+orderbacks.ScanCode+"\" style=\"width: 350px; height:350px;\"/>")
				} else if closebacks.AppParams != "" { //返回APP字符串不为空
					fmt.Fprintf(w, "返回APP字符串内容\n"+closebacks.AppParams)
					//fmt.Fprintf(w,"{\"type\":\"app\",\"val\":\"" + orderbacks.AppParams + "\"}")
				} else {
					fmt.Fprintf(w, "成功返回内容\n"+response)
				}
			} else {
				fmt.Println("失败:", status)
				fmt.Fprintf(w, "失败:"+status)
			}
		} else {
			fmt.Println("verr:", verr)
			fmt.Fprintf(w, "验签异常!")
		}
	} else {
		fmt.Println("response:", response)
		fmt.Fprintf(w, response)
	}
}

// 同步返回内容处理
func closeCallback(w http.ResponseWriter, r *http.Request) {
	r.ParseForm()
	merchantId := utils.IsNull(r.Form["merchantId"])
	requestId := utils.IsNull(r.Form["requestId"])
	fmt.Fprintf(w, "商户号："+merchantId)
	fmt.Fprintf(w, "\n订单号："+requestId)
	return
}
