package gomod

import (
	"Geoswift_No_Reason_Remittance_Golang/models"
	"Geoswift_No_Reason_Remittance_Golang/utils"
	"encoding/json"
	"fmt"
	"net/http"
	"path/filepath"
)

// 订单查询请求
func RateQueryRequest(w http.ResponseWriter, r *http.Request) {
	//接口地址
	apiurl := "https://apis.geoswift.com/overseas/listPrice/bid"

	//pfx私钥
	privatepfxpath, pfxerr := filepath.Abs("package/JHclient.pfx")
	if pfxerr != nil {
		fmt.Println("pfxerr:", pfxerr)
	}
	pfxpwd := "1234"
	//cer公钥
	publiccerpath, cererr := filepath.Abs("package/server.cer")
	if cererr != nil {
		fmt.Println("cererr:", cererr)
	}
	var aesKeyValue string = utils.GetRandomString(16)
	r.ParseForm()
	var rateQueryJson models.RateQuery
	rateQueryJson.MerchantId = utils.IsNull(r.Form["merchantId"])
	rateQueryJson.SourceCurrency = utils.IsNull(r.Form["sourceCurrency"])
	rateQueryJson.BusinessType = utils.IsNull(r.Form["businessType"])
	//把订单对象数据转JSON字符串
	orderdata, err := json.Marshal(rateQueryJson)
	if err != nil {
		fmt.Print(err.Error())
		return
	}
	sortdata, oerr := utils.GetMapSortJson(string(orderdata))
	if oerr != nil {
		fmt.Println("oerr:", oerr)
	}
	hm := utils.Sha1(sortdata)
	hmac, err := utils.Sign(hm, privatepfxpath, pfxpwd)
	if err != nil {
		fmt.Println("err:", err)
	}
	rateQueryJson.Hmac = hmac
	orderjson, err := json.Marshal(rateQueryJson)
	if err != nil {
		fmt.Print(err.Error())
		return
	}
	encryptKey, enerr := utils.PublicEncrypt(aesKeyValue, publiccerpath)
	if enerr != nil {
		fmt.Println("err:", enerr)
	}
	aesdata, aeserro := utils.AesECBEncrypt(string(orderjson), aesKeyValue)
	//fmt.Println("aesdata:", aesdata)
	if aeserro != nil {
		fmt.Println("err:", aeserro)
	}
	response, rperr := utils.Post2(apiurl, encryptKey, rateQueryJson.MerchantId, aesdata, privatepfxpath, pfxpwd)
	if rperr != nil {
		fmt.Println("rperr:", rperr)
	}
	var queryorderback models.QueryOrderBack
	json.Unmarshal([]byte(response), &queryorderback)
	rhmac := queryorderback.Hmac
	//fmt.Println("queryorder:", queryorderback)
	if rhmac != "" {
		sortdeldata, oerr := utils.GetDelJson(string(response), "hmac")
		if oerr != nil {
			fmt.Println("oerr:", oerr)
		}
		signhm := utils.Sha1(sortdeldata)
		verr := utils.VerifySign(signhm, publiccerpath, rhmac)
		if verr == nil {
			status := queryorderback.Status
			if status == "SUCCESS" {
				//fmt.Println("成功:", response)
				fmt.Fprintf(w, "订单状态成功\n"+response)
			} else {
				//fmt.Println("异常:", response)
				fmt.Fprintf(w, "订单状态异常\n"+response)
			}
		} else {
			fmt.Println("verr:", verr)
		}
	} else {
		fmt.Println("response:", response)
	}
}
