package gomod

import (
	"Geoswift_No_Reason_Remittance_Golang/models"
	"Geoswift_No_Reason_Remittance_Golang/utils"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net/http"
	"path/filepath"
)

func RemitRequest(w http.ResponseWriter, r *http.Request) {
	//接口地址
	apiurl := "https://apis.geoswift.com/remittance/order"
	//pfx私钥
	privatepfxpath, pfxerr := filepath.Abs("package/JHclient.pfx")
	if pfxerr != nil {
		fmt.Println("pfxerr:", pfxerr)
	}
	pfxpwd := "1234"
	//cer公钥
	publiccerpath, cererr := filepath.Abs("package/server.cer")
	if cererr != nil {
		fmt.Println("cererr:", cererr)
	}
	//生成16位随机数
	var aesKeyValue string = utils.GetRandomString(16)
	r.ParseForm()
	var payorder models.RemitSubmit
	payorder.MerchantId = utils.IsNull(r.Form["merchantId"])
	payorder.RequestId = utils.IsNull(r.Form["requestId"])
	payorder.RequestCreditedAmount = utils.IsNull(r.Form["requestCreditedAmount"])
	payorder.Amount = utils.IsNull(r.Form["amount"])
	payorder.Currency = utils.IsNull(r.Form["currency"])
	payorder.NotifyUrl = utils.IsNull(r.Form["notifyUrl"])
	payorder.TradePostscript = utils.IsNull(r.Form["tradePostscript"])
	payorder.SourceCountry = utils.IsNull(r.Form["sourceCountry"])
	payorder.FundPurpose = utils.IsNull(r.Form["fundPurpose"])

	//List
	//var gds []models.PayeeAccount
	//err := json.Unmarshal([]byte(utils.IsNull(r.Form["payeeAccount"])), &gds)
	//if err != nil {
	//	fmt.Println("jsonerror:", err)
	//}
	//payorder.PayeeAccount = gds

	var payeeAccount models.PayeeAccount
	payeeAccount.AccountType = utils.IsNull(r.Form["accountType"])
	payeeAccount.OrganCode = utils.IsNull(r.Form["organCode"])
	payeeAccount.WalletAccountNo = utils.IsNull(r.Form["walletAccountNo"])
	payorder.PayeeAccount = payeeAccount

	//对象
	var payee models.Payee
	payee.Name = utils.IsNull(r.Form["payeeName"])
	payee.IdType = utils.IsNull(r.Form["payeeIdType"])
	payee.IdNum = utils.IsNull(r.Form["payeeIdNum"])
	payee.PhoneNum = utils.IsNull(r.Form["payeePhoneNum"])
	payee.BankCardNum = utils.IsNull(r.Form["payeeBankCardNum"])
	payee.BankName = utils.IsNull(r.Form["payeeBankName"])
	payee.BankAddress = utils.IsNull(r.Form["payeeBankAddress"])
	payee.BankCity = utils.IsNull(r.Form["payeeBankCity"])
	payee.PayeeBirthday = utils.IsNull(r.Form["payeeBirthday"])
	payorder.Payee = payee

	var payerCollect models.Payer
	payerCollect.Name = utils.IsNull(r.Form["payerName"])
	payerCollect.IdType = utils.IsNull(r.Form["payerIdType"])
	payerCollect.IdNum = utils.IsNull(r.Form["payerIdNum"])
	payerCollect.BankCardNum = utils.IsNull(r.Form["payerBankCardNum"])
	payerCollect.PayerCountry = utils.IsNull(r.Form["payerCountry"])
	payerCollect.Address = utils.IsNull(r.Form["payerAddress"])
	payerCollect.Birthday = utils.IsNull(r.Form["payerBirthday"])
	payerCollect.SwiftCode = utils.IsNull(r.Form["payerSwiftCode"])
	payerCollect.BankName = utils.IsNull(r.Form["payerBankName"])
	payerCollect.BankAddress = utils.IsNull(r.Form["payerBankAddress"])
	payerCollect.BankCity = utils.IsNull(r.Form["payerBankCity"])
	payerCollect.PayerCity = utils.IsNull(r.Form["payerCity"])
	payerCollect.PayerNameZh = utils.IsNull(r.Form["payerNameZh"])
	payerCollect.PayerGender = utils.IsNull(r.Form["payerGender"])
	payorder.Payer = payerCollect
	//组织数据完毕

	//把订单对象数据转JSON字符串
	orderdata, err := json.Marshal(payorder)
	if err != nil {
		fmt.Printf(err.Error())
		return
	}
	sortdata, oerr := utils.GetMapSortJson(string(orderdata))
	if oerr != nil {
		fmt.Println("oerr:", oerr)
	}
	hm := utils.Sha1(sortdata)
	hmac, err := utils.Sign(hm, privatepfxpath, pfxpwd)
	if err != nil {
		fmt.Println("err:", err)
	}
	payorder.Hmac = hmac
	orderjson, err := json.Marshal(payorder)
	if err != nil {
		fmt.Printf(err.Error())
		return
	}
	encryptKey, enerr := utils.PublicEncrypt(aesKeyValue, publiccerpath)
	if enerr != nil {
		fmt.Println("err:", enerr)
	}
	aesdata, aeserro := utils.AesECBEncrypt(string(orderjson), aesKeyValue)
	//fmt.Println("aesdata:", aesdata)
	if aeserro != nil {
		fmt.Println("err:", aeserro)
	}
	response, rperr := utils.Post(apiurl, encryptKey, payorder.MerchantId, payorder.RequestId, aesdata, privatepfxpath, pfxpwd)
	if rperr != nil {
		fmt.Println("rperr:", rperr)
	}
	var orderbacks models.RemitSubmitBacks
	json.Unmarshal([]byte(response), &orderbacks)
	rhmac := orderbacks.Hmac
	//fmt.Println("rhmac:", rhmac)
	if rhmac != "" {
		sortdeldata, oerr := utils.GetDelJson(string(response), "hmac")
		if oerr != nil {
			fmt.Println("oerr:", oerr)
		}
		signhm := utils.Sha1(sortdeldata)
		verr := utils.VerifySign(signhm, publiccerpath, rhmac)
		if verr == nil {
			status := orderbacks.Status
			if status == "REDIRECT" {
				w.Header().Set("Location", orderbacks.RedirectUrl) //跳转地址设置
				w.WriteHeader(301)
				http.Redirect(w, r, orderbacks.RedirectUrl, http.StatusFound)
				return
			} else if status == "SUCCESS" {
				//返回扫码Base64不为空
				if orderbacks.ScanCodeUrl != "" {
					//fmt.Println("response:", response)
					//fmt.Println("orderbacks.ScanCode:", orderbacks.ScanCode)
					fmt.Fprintf(w, "{\"type\":\"code\",\"val\":\""+orderbacks.ScanCodeUrl+"\"}")
					//w.Header().Set("Content-Type", "text/html")
					//fmt.Fprintf(w,"请扫描下方二维码完成支付：<br/>")
					//fmt.Fprintf(w, "<img id=\"cordbase64\" src=\"data:image/png;base64,"+orderbacks.ScanCode+"\" style=\"width: 350px; height:350px;\"/>")
				} else if orderbacks.AppParams != "" { //返回APP字符串不为空
					fmt.Fprintf(w, "返回APP字符串内容\n"+orderbacks.AppParams)
					//fmt.Fprintf(w,"{\"type\":\"app\",\"val\":\"" + orderbacks.AppParams + "\"}")
				} else {
					fmt.Fprintf(w, "成功返回内容\n"+response)
				}
			} else {
				fmt.Println("失败:", status)
				fmt.Fprintf(w, "失败:"+status)
			}
		} else {
			fmt.Println("verr:", verr)
			fmt.Fprintf(w, "验签异常!")
		}
	} else {
		fmt.Println("response:", response)
		fmt.Fprintf(w, response)
	}
}

// 同步返回内容处理
func OrderCallback(w http.ResponseWriter, r *http.Request) {
	r.ParseForm()
	merchantId := utils.IsNull(r.Form["merchantId"])
	requestId := utils.IsNull(r.Form["requestId"])
	//io.WriteString(w, "商户号："+merchantId)
	fmt.Fprintf(w, "商户号："+merchantId)
	fmt.Fprintf(w, "\n订单号："+requestId)
	return
}

// 异步返回内容处理
func OrderNotify(w http.ResponseWriter, r *http.Request) {
	//pfx私钥
	privatepfxpath, pfxerr := filepath.Abs("src/payease_onlinepay/package/JHclient.pfx")
	if pfxerr != nil {
		fmt.Println("pfxerr:", pfxerr)
	}
	pfxpwd := "123456"
	//cer公钥
	publiccerpath, cererr := filepath.Abs("src/payease_onlinepay/package/server.cer")
	if cererr != nil {
		fmt.Println("cererr:", cererr)
	}
	r.ParseForm()
	rpencryptKey := utils.IsNull(r.Header["Encryptkey"])
	if rpencryptKey != "" {
		aeskey, err := utils.PrivateDecrypt(rpencryptKey, privatepfxpath, pfxpwd)
		if err != nil {
			fmt.Println("perr:", err)
		}
		body, err := ioutil.ReadAll(r.Body)
		if err != nil {
			fmt.Println("berr:", err)
		}
		var rback utils.ResponseBack
		json.Unmarshal(body, &rback)
		ret, err := utils.AesECBDecrypt(rback.Data, aeskey)
		if err != nil {
			fmt.Println("aerr:", err)
		}
		var orderbacks models.RemitSubmitBacks
		json.Unmarshal([]byte(ret), &orderbacks)
		rhmac := orderbacks.Hmac
		//fmt.Println("rhmac:", rhmac)
		if rhmac != "" {
			sortdeldata, oerr := utils.GetDelJson(string(ret), "hmac")
			if oerr != nil {
				fmt.Println("oerr:", oerr)
			}
			signhm := utils.Sha1(sortdeldata)
			verr := utils.VerifySign(signhm, publiccerpath, rhmac)
			if verr == nil {
				status := orderbacks.Status
				if status == "SUCCESS" {
					fmt.Fprintf(w, "SUCCESS")
					fmt.Println("SUCCESS")
					fmt.Println("处理成功!")
					fmt.Println("ret:", ret)
				} else if status == "FAILED" {
					fmt.Println("处理失败:", status)
					fmt.Println("ret:", ret)
				} else if status == "INIT" {
					fmt.Println("待处理:", status)
					fmt.Println("ret:", ret)
				} else {
					fmt.Println("处理异常:", status)
					fmt.Println("ret:", ret)
				}
			} else {
				fmt.Println("verr:", verr)
			}
		} else {
			fmt.Println("response:", ret)
		}
	} else {
		fmt.Println("NO Data")
	}
	return
}
