package models

// Submit transaction
type RemitSubmit struct {
	MerchantId            string       `json:"merchantId"`
	RequestId             string       `json:"requestId"`
	RequestCreditedAmount string       `json:"requestCreditedAmount"`
	Amount                string       `json:"amount"`
	Currency              string       `json:"currency"`
	NotifyUrl             string       `json:"notifyUrl"`
	TradePostscript       string       `json:"tradePostscript"`
	SourceCountry         string       `json:"sourceCountry"`
	FundPurpose           string       `json:"fundPurpose"`
	Payee                 Payee        `json:"payee"`
	Payer                 Payer        `json:"payer"`
	PayeeAccount          PayeeAccount `json:"payeeAccount"`
	Hmac                  string       `json:"hmac"`
}

// 下单响应数据
type RemitSubmitBacks struct {
	MerchantId            string `json:"merchantId"`
	RequestId             string `json:"requestId"`
	SerialNumber          string `json:"serialNumber"`
	Amount                string `json:"amount"`
	RequestCreditedAmount string `json:"requestCreditedAmount"`
	PayerCountry          string `json:"payerCountry"`
	Currency              string `json:"currency"`
	TradePostscript       string `json:"tradePostscript"`
	Status                string `json:"status"`
	BusinessStatus        string `json:"businessStatus"`
	FeeAmount             string `json:"feeAmount"`
	FeeCurrency           string `json:"feeCurrency"`
	CreditedAmount        string `json:"creditedAmount"`
	CreditedCurrency      string `json:"creditedCurrency"`
	Rate                  string `json:"rate"`
	CompleteDateTime      string `json:"completeDateTime"`
	ErrorMsg              string `json:"errorMsg"`
	ResponseObject
}

// Payee
type Payee struct {
	Name                   string `json:"name"`
	NameZh                 string `json:"nameZh"`
	SwiftCode              string `json:"swiftCode"`
	IdType                 string `json:"idType"`
	IdNum                  string `json:"idNum"`
	PhoneNum               string `json:"phoneNum"`
	BankCardNum            string `json:"bankCardNum"`
	BankName               string `json:"bankName"`
	BankAddress            string `json:"bankAddress"`
	BankCity               string `json:"bankCity"`
	PayeeIdEffectDateStart string `json:"payeeIdEffectDateStart"`
	PayeeIdEffectDateEnd   string `json:"payeeIdEffectDateEnd"`
	PayeeAddress           string `json:"payeeAddress"`
	PayeeAreaCode          string `json:"payeeAreaCode"`
	PayeeProfessionCode    string `json:"payeeProfessionCode"`
	PayeeGender            string `json:"payeeGender"`
	FirstName              string `json:"firstName"`
	LastName               string `json:"lastName"`
	PayeeBirthday          string `json:"payeeBirthday"`
}

// Payer
type Payer struct {
	Name         string `json:"name"`
	IdType       string `json:"idType"`
	IdNum        string `json:"idNum"`
	BankCardNum  string `json:"bankCardNum"`
	PayerCountry string `json:"payerCountry"`
	Address      string `json:"address"`
	Birthday     string `json:"birthday"`
	SwiftCode    string `json:"swiftCode"`
	BankName     string `json:"bankName"`
	BankAddress  string `json:"bankAddress"`
	BankCity     string `json:"bankCity"`
	FirstName    string `json:"firstName"`
	LastName     string `json:"lastName"`
	PayerCity    string `json:"payerCity"`
	PayerNameZh  string `json:"payerNameZh"`
	PayerGender  string `json:"payerGender"`
}

// PayeeAccount
type PayeeAccount struct {
	AccountType     string `json:"accountType"`
	OrganCode       string `json:"organCode"`
	WalletAccountNo string `json:"walletAccountNo"`
}

// HotelInfo
type HotelInfo struct {
	HotelName    string `json:"hotelName"`
	CheckInDate  string `json:"checkInDate"`
	CheckOutDate string `json:"checkOutDate"`
}

// OtherInfo
type OtherInfo struct {
	Description string `json:"description"`
}

// 提交查询订单信息
type QueryOrder struct {
	MerchantId string `json:"merchantId"`
	RequestId  string `json:"requestId"`
	PartnerId  string `json:"partnerId"`
	Hmac       string `json:"hmac"`
}

// 查询订单响应内容
type QueryOrderBack struct {
	MerchantId         string `json:"merchantId"`
	PartnerId          string `json:"partnerId"`
	RequestId          string `json:"requestId"`
	SerialNumber       string `json:"serialNumber"`
	BankRequestNumber  string `json:"bankRequestNumber"`
	TotalRefundCount   string `json:"totalRefundCount"`
	TotalRefundAmount  string `json:"totalRefundAmount"`
	OrderCurrency      string `json:"orderCurrency"`
	OrderAmount        string `json:"orderAmount"`
	SettlementAmount   string `json:"settlementAmount"`
	SettlementCurrency string `json:"settlementCurrency"`
	ExchangeRate       string `json:"exchangeRate"`

	FeeAmount        string `json:"feeAmount"`
	FeeCurrency      string `json:"feeCurrency"`
	GatewayType      string `json:"gatewayType"`
	CompleteDateTime string `json:"completeDateTime"`
	PaymentModeCode  string `json:"paymentModeCode"`
	Remark           string `json:"remark"`
	NeedReview       string `json:"needReview"`
	ResponseObject
}

// 提交退款订单信息
type RefundOrder struct {
	MerchantId   string `json:"merchantId"`
	PartnerId    string `json:"partnerId"`
	RequestId    string `json:"requestId"`
	RefundAmount string `json:"refundAmount"`
	OrderId      string `json:"orderId"`
	NotifyUrl    string `json:"notifyUrl"`
	Remark       string `json:"remark"`
	Hmac         string `json:"hmac"`
}

// 退款订单响应内容
type RefundOrderBack struct {
	MerchantId        string `json:"merchantId"`
	PartnerId         string `json:"partnerId"`
	RequestId         string `json:"requestId"`
	BankRequestNumber string `json:"bankRequestNumber"`
	SerialNumber      string `json:"serialNumber"`
	Amount            string `json:"amount"`
	Currency          string `json:"currency"`
	Remark            string `json:"remark"`
	ResponseObject
}

// RateQuery
type RateQuery struct {
	MerchantId     string `json:"merchantId"`
	SourceCurrency string `json:"sourceCurrency"`
	BusinessType   string `json:"businessType"`
	Hmac           string `json:"hmac"`
}

type RateQueryBack struct {
	MerchantId     string `json:"merchantId"`
	SourceCurrency string `json:"sourceCurrency"`
	TargetCurrency string `json:"targetCurrency"`
	Listprice      string `json:"listprice"`
	TradeUnit      string `json:"tradeUnit"`
	Status         string `json:"status"`
	ResMessage     string `json:"resMessage"`
	Hmac           string `json:"hmac"`
}

// Cancel Transaction
type Close struct {
	MerchantId          string `json:"merchantId"`
	RequestId           string `json:"requestId"`
	RemittanceRequestId string `json:"remittanceRequestId"`
	ProxyRequestId      string `json:"proxyRequestId"`
	MerchantReference   string `json:"merchantReference"`
	Remark              string `json:"remark"`
	Hmac                string `json:"hmac"`
}

// 订单关闭响应内容
type CloseBack struct {
	MerchantId          string `json:"merchantId"`
	RequestId           string `json:"requestId"`
	RemittanceRequestId string `json:"remittanceRequestId"`
	ProxyRequestId      string `json:"proxyRequestId"`
	MerchantReference   string `json:"merchantReference"`
	StatusInfo          string `json:"status"`
	CancelStatus        string `json:"cancelStatus"`
	CreateDateTime      string `json:"createDateTime"`
	CompleteDateTime    string `json:"completeDateTime"`
	Remark              string `json:"remark"`
	ErrorCode           string `json:"errorCode"`
	ErrorMsg            string `json:"errorMsg"`
	ResponseObject
}
